"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteBatchScheduleExecution = exports.SaveBatchScheduleExecution = exports.GetBatchScheduleExecutionsByBatchType = exports.GetBatchScheduleExecutionsByCriteria = exports.DeleteBatchSchedule = exports.SaveBatchSchedule = exports.GetBatchScheduleById = exports.GetBatchSchedulesByCriteria = exports.GetBatchSchedulesSummary = exports.BatchScheduleMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function BatchScheduleMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.BatchScheduleMSSql = BatchScheduleMSSql;
function GetBatchSchedulesSummary(vcc, searchCriteria, searchExpression) {
    var recordset = null;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(500), searchExpression)
        .input('BatchType', sql.VarChar(50), 'TRANSCRIPTION')
        .execute('GetBatchSchedulesSummary')
        .then(function (recordsets) {
        recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[BatchScheduleMSSql::GetBatchSchedulesByCriteria] getting schedules recordset:', recordset.length);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchSchedulesByCriteria] getting errorMsg:', errorMsg, 'of GetBatchSchedules');
        return Promise.reject(err);
    });
}
exports.GetBatchSchedulesSummary = GetBatchSchedulesSummary;
function GetBatchSchedulesByCriteria(vcc, searchCriteria, searchExpression, extendedContent) {
    if (extendedContent === void 0) { extendedContent = false; }
    var recordset = null;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(500), searchExpression)
        .input('ExtendedContent', sql.Bit, extendedContent)
        .execute('GetBatchSchedules')
        .then(function (recordsets) {
        recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[BatchScheduleMSSql::GetBatchSchedulesByCriteria] getting schedules recordset:', recordset.length);
        if (recordset.length && extendedContent) {
            recordset.map(function (r) {
                if (r.batchType === 'TRANSCRIPTION')
                    r.batchData = JSON.parse(r.batchData);
            });
        }
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchSchedulesByCriteria] getting errorMsg:', errorMsg, 'of GetBatchSchedules');
        return Promise.reject(err);
    });
}
exports.GetBatchSchedulesByCriteria = GetBatchSchedulesByCriteria;
function GetBatchScheduleById(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), id)
        .execute('GetBatchScheduleById')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var process = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(process);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetBatchScheduleById = GetBatchScheduleById;
function SaveBatchSchedule(vcc, scheduleData, userId) {
    var saveBatchScheduleResult = false;
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), scheduleData.id)
        .input('BatchType', sql.VarChar(50), scheduleData.batchType)
        .input('Frequency', sql.VarChar(50), scheduleData.frequency)
        .input('Weekday', sql.Int, scheduleData.weekDay)
        .input('Monthday', sql.Int, scheduleData.monthDay)
        .input('DaysInterval', sql.Int, scheduleData.daysInterval)
        .input('HoursInterval', sql.Int, scheduleData.hoursInterval)
        .input('ExecutionHour', sql.VarChar(50), scheduleData.executionHour)
        .input('FirstExecutionDate', sql.DateTime, scheduleData.firstExecutionDate)
        .input('ExecutionDelay', sql.Int, scheduleData.executionDelay)
        .input('StartDate', sql.DateTime, scheduleData.startDate)
        .input('EndDate', sql.DateTime, scheduleData.endDate)
        .input('BatchData', sql.VarChar(sql.MAX), JSON.stringify(scheduleData.batchData))
        .input('ProcessFiltersId', sql.VarChar(50), scheduleData.processFiltersId)
        .input('Status', sql.VarChar(50), scheduleData.status)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveBatchSchedule')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        saveBatchScheduleResult = output && output.result === 1 ? true : false;
        var promiseArray = [];
        if (scheduleData.executionList && scheduleData.executionList.length)
            scheduleData.executionList.map(function (el) { return promiseArray.push(SaveBatchScheduleExecution(vcc, el, userId)); });
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        return Promise.resolve(saveBatchScheduleResult);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing error ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveBatchSchedule = SaveBatchSchedule;
function DeleteBatchSchedule(vcc, batchScheduleId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), batchScheduleId)
        .execute('DeleteBatchSchedule')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteBatchSchedule = DeleteBatchSchedule;
function GetBatchScheduleExecutionsByCriteria(vcc, searchCriteria, searchExpression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), searchCriteria)
        .input('SearchExpression', sql.VarChar(500), searchExpression)
        .execute('GetBatchScheduleExecutionsByCriteria')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : []);
        logger.info('[BatchScheduleMSSql::GetBatchScheduleExecutionsByCriteria] getting schedules recordset:', recordset.length);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchScheduleExecutionsByCriteria] getting errorMsg:', errorMsg, 'of GetBatchScheduleExecutionsByCriteria');
        return Promise.reject(err);
    });
}
exports.GetBatchScheduleExecutionsByCriteria = GetBatchScheduleExecutionsByCriteria;
function GetBatchScheduleExecutionsByBatchType(vcc, batchScheduleId, batchType) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('BatchScheduleId', sql.VarChar(50), batchScheduleId)
        .input('BatchType', sql.VarChar(500), batchType)
        .execute('GetBatchScheduleExecutionsByBatchType')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : []);
        logger.info('[BatchScheduleMSSql::GetBatchScheduleExecutionsByBatchType] getting schedules recordset:', recordset.length);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::GetBatchScheduleExecutionsByBatchType] getting errorMsg:', errorMsg, 'of GetBatchScheduleExecutionsByCriteria');
        return Promise.reject(err);
    });
}
exports.GetBatchScheduleExecutionsByBatchType = GetBatchScheduleExecutionsByBatchType;
function SaveBatchScheduleExecution(vcc, scheduleExecution, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Id', sql.VarChar(50), scheduleExecution.id)
        .input('BatchScheduleId', sql.VarChar(50), scheduleExecution.batchScheduleId)
        .input('BatchId', sql.VarChar(50), scheduleExecution.batchId)
        .input('ExecutionDate', sql.DateTime, scheduleExecution.executionDate)
        .input('Order', sql.Int, scheduleExecution.order)
        .input('Status', sql.VarChar(50), scheduleExecution.status)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveBatchScheduleExecution')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Executing error ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveBatchScheduleExecution = SaveBatchScheduleExecution;
function DeleteBatchScheduleExecution(vcc, scheduleExecutionId) {
    logger.info('[IntentMSSql::DeleteEntity] getting vcc:', vcc, 'executionId:', scheduleExecutionId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(50), scheduleExecutionId)
        .execute('DeleteBatchScheduleExecution')
        .then(function (recordsets) {
        logger.info('[BatchScheduleMSSql::DeleteBatchScheduleExecution] getting recordsets:', recordsets, 'of DeleteBatchScheduleExecution');
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[BatchScheduleMSSql::DeleteBatchScheduleExecution] getting errorMsg:', errorMsg, 'of DeleteEntity');
        return Promise.reject(err);
    });
}
exports.DeleteBatchScheduleExecution = DeleteBatchScheduleExecution;
