"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteSubscriptionKey = exports.GetSubscriptionKey = exports.SaveSubscriptionKey = exports.SaveUserAccessTokenByRealm = exports.GetUserAccessTokenByRealm = exports.RemoveInteractionAudioTransfer = exports.RemoveInteraction = exports.GetInteractionsTranscriptions = exports.GetInteractionsAudioTransfer = exports.GetInteractions = exports.SaveInteractionsTranscriptions = exports.SaveInteractionsAudioTransfer = exports.SaveInteraction = exports.GetInteractionsSearch = exports.SaveInteractionSearch = exports.InteractionRedis = void 0;
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function InteractionRedis(app) {
    redisClient = app.get('redisClient');
}
exports.InteractionRedis = InteractionRedis;
function SaveInteractionSearch(key, interactions) {
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync('Interaction', key, JSON.stringify(interactions))
        .then(function (result) {
        return redisClient.expireAsync(key + ':Interaction', redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        return Promise.reject(err);
    });
}
exports.SaveInteractionSearch = SaveInteractionSearch;
function GetInteractionsSearch(key) {
    return redisClient.hgetAsync('Interaction', key)
        .then(function (result) {
        var interaction = [];
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch(function (err) {
        return Promise.reject(err);
    });
}
exports.GetInteractionsSearch = GetInteractionsSearch;
function SaveInteraction(processInteraction) {
    var key = 'interactions:';
    key += processInteraction.vcc;
    key += ':';
    key += processInteraction.processId;
    key += ':';
    key += processInteraction.id;
    if (processInteraction.section !== undefined) {
        key += ':';
        key += processInteraction.section;
    }
    if (processInteraction.part) {
        key += ':';
        key += processInteraction.part;
    }
    var storage = 'interactions';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(processInteraction))
        .then(function (result) {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::SaveInteraction] getting error:', err, 'with vcc:', processInteraction.vcc, 'processId:', processInteraction.processId, 'processInteraction:', processInteraction, 'with storage:', storage);
        return Promise.resolve(err);
    });
}
exports.SaveInteraction = SaveInteraction;
function SaveInteractionsAudioTransfer(processInteraction) {
    var storage = 'interactionsAudioTransfer';
    var key = 'interactionsAudioTransfer:';
    key += processInteraction.vcc;
    key += ':';
    key += processInteraction.processId;
    key += ':';
    key += processInteraction.id;
    if (processInteraction.section !== undefined) {
        key += ':';
        key += processInteraction.section;
    }
    if (processInteraction.part) {
        key += ':';
        key += processInteraction.part;
    }
    return redisClient.hsetAsync(storage, key, JSON.stringify(processInteraction))
        .then(function (result) {
        return redisClient.expireAsync(processInteraction.processId + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(processInteraction);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::SaveInteractionsAudioTransfer] getting error:', err, 'with vcc:', processInteraction.vcc, 'processId:', processInteraction.processId, 'processInteraction:', processInteraction, 'with storage:', storage);
        return Promise.resolve(err);
    });
}
exports.SaveInteractionsAudioTransfer = SaveInteractionsAudioTransfer;
function SaveInteractionsTranscriptions(processInteraction) {
    var storage = 'interactionsTranscriptions';
    var key = 'interactionsTranscriptions:';
    key += processInteraction.vcc;
    key += ':';
    key += processInteraction.processId;
    key += ':';
    key += processInteraction.id;
    if (processInteraction.section !== undefined) {
        key += ':';
        key += processInteraction.section;
    }
    if (processInteraction.part) {
        key += ':';
        key += processInteraction.part;
    }
    return redisClient.hsetAsync(storage, key, JSON.stringify(processInteraction))
        .then(function (result) {
        return redisClient.expireAsync(processInteraction.processId + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(processInteraction);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::SaveInteractionsTranscriptions] getting error:', err, 'with vcc:', processInteraction.vcc, 'processId:', processInteraction.processId, 'processInteraction data:', processInteraction, 'with storage:', storage);
        return Promise.resolve(err);
    });
}
exports.SaveInteractionsTranscriptions = SaveInteractionsTranscriptions;
function GetInteractions() {
    return redisClient.hgetallAsync('interactions')
        .then(function (result) {
        var interactionProcessToSendList = [];
        try {
            var interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (var key in interactionsToSendResults) {
                    interactionProcessToSendList.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[InteractionRedis::GetInteractions] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactionProcessToSendList;
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::GetInteractions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetInteractions = GetInteractions;
function GetInteractionsAudioTransfer() {
    return redisClient.hgetallAsync('interactionsAudioTransfer')
        .then(function (result) {
        var interactionProcessToSendList = [];
        try {
            var interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (var key in interactionsToSendResults) {
                    interactionProcessToSendList.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[InteractionRedis::GetInteractionsAudioTransfer] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactionProcessToSendList;
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::GetInteractionsAudioTransfer] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetInteractionsAudioTransfer = GetInteractionsAudioTransfer;
function GetInteractionsTranscriptions() {
    return redisClient.hgetallAsync('interactionsTranscriptions')
        .then(function (result) {
        var interactionProcessToSendList = [];
        try {
            var interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (var key in interactionsToSendResults) {
                    interactionProcessToSendList.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[InteractionRedis::GetInteractionsTranscriptions] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactionProcessToSendList;
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::GetInteractionsTranscriptions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetInteractionsTranscriptions = GetInteractionsTranscriptions;
function RemoveInteraction(processInteraction) {
    var key = 'interactions:';
    key += processInteraction.vcc;
    key += ':';
    key += processInteraction.processId;
    key += ':';
    key += processInteraction.id;
    if (processInteraction.section !== undefined) {
        key += ':';
        key += processInteraction.section;
    }
    if (processInteraction.part) {
        key += ':';
        key += processInteraction.part;
    }
    var storage = 'interactions';
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::RemoveInteraction] getting error:', err, 'with vcc:', processInteraction.vcc, 'processId:', processInteraction.processId, 'processInteraction data:', processInteraction, 'with storage:', storage);
        return Promise.resolve(err);
    });
}
exports.RemoveInteraction = RemoveInteraction;
function RemoveInteractionAudioTransfer(processInteraction) {
    var key = 'interactionsAudioTransfer:';
    key += processInteraction.vcc;
    key += ':';
    key += processInteraction.processId;
    key += ':';
    key += processInteraction.id;
    if (processInteraction.section !== undefined) {
        key += ':';
        key += processInteraction.section;
    }
    if (processInteraction.part) {
        key += ':';
        key += processInteraction.part;
    }
    var storage = 'interactionsAudioTransfer';
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::RemoveInteractionAudioTransfer] getting error:', err, 'with vcc:', processInteraction.vcc, 'processId:', processInteraction.processId, 'processInteraction data:', processInteraction, 'with storage:', storage);
        return Promise.resolve(err);
    });
}
exports.RemoveInteractionAudioTransfer = RemoveInteractionAudioTransfer;
function GetUserAccessTokenByRealm(key) {
    var hash = 'realm:';
    hash += key;
    return redisClient.hgetAsync(hash, key)
        .then(function (result) {
        var userAcessToken;
        try {
            userAcessToken = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return userAcessToken;
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::GetUserAccessTokenByRealm] getting err value:', err, 'hash:', hash, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetUserAccessTokenByRealm = GetUserAccessTokenByRealm;
function SaveUserAccessTokenByRealm(key, userAcessToken) {
    var hash = 'realm:';
    hash += key;
    if (!hash)
        return Promise.resolve(false);
    return redisClient.hsetAsync(hash, key, typeof userAcessToken === 'object' ? JSON.stringify(userAcessToken) : userAcessToken)
        .then(function (result) {
        return redisClient.expireAsync(hash + ':' + hash, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::SaveUserAccessTokenByRealm] getting error:', err, 'with vcc:', userAcessToken.vcc, 'userName:', userAcessToken.userName, 'realm:', userAcessToken.realm, 'with hash:', hash);
        return Promise.resolve(false);
    });
}
exports.SaveUserAccessTokenByRealm = SaveUserAccessTokenByRealm;
function SaveSubscriptionKey(key, azure) {
    var hash = 'azure';
    if (!hash)
        return Promise.resolve(false);
    return redisClient.hsetAsync(hash, key, typeof azure === 'object' ? JSON.stringify(azure) : azure)
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::SaveUserAccessTokenByRealm] getting error:', err, 'with vcc:', key);
        return Promise.resolve(false);
    });
}
exports.SaveSubscriptionKey = SaveSubscriptionKey;
function GetSubscriptionKey(key) {
    var hash = 'azure';
    return redisClient.hgetAsync(hash, key)
        .then(function (result) {
        var credential;
        try {
            credential = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return credential;
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::GetUserAccessTokenByRealm] getting err value:', err, 'hash:', hash, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetSubscriptionKey = GetSubscriptionKey;
function DeleteSubscriptionKey(vcc) {
    var storage = 'azure';
    if (!vcc)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, vcc)
        .then(function (result) {
        return Promise.resolve(result ? true : false);
    })
        .catch(function (err) {
        logger.error('[InteractionRedis::DeleteSubscriptionKey] getting error:', err, 'with key:', vcc);
        return Promise.resolve(null);
    });
}
exports.DeleteSubscriptionKey = DeleteSubscriptionKey;
