"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoveEvaluationPlanProcess = exports.CreateQueueL = exports.SaveAuditorEP = exports.SaveEvaluationPlan = exports.EvaluationPlanService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var evaluation_plan_model_1 = require("../models/evaluation-plan.model");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var bluebird_1 = require("bluebird");
var linguistic_service_1 = require("./linguistic.service");
var billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
var billing_implementation_1 = require("../implementation/billing.implementation");
var interaction_model_1 = require("../models/interaction.model");
var interaction_service_1 = require("./interaction.service");
var billing_model_1 = require("../models/billing.model");
var platform_service_1 = require("./platform.service");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var html_to_pdf = require('html-pdf');
var logger = log4js.getLogger('ServerScripts');
function EvaluationPlanService(app) {
    var _this = this;
    app.post('/api/evaluationplan/pdf/', function (req, res) {
        var htmlBuffer = new Buffer(req.body.html, 'base64');
        var uriBuffer = new Buffer(req.body.uri, 'base64');
        var html = htmlBuffer.toString("ascii");
        var uri = uriBuffer.toString("ascii");
        var content = "<!doctype html>\n\t\t<html>\n\t\t   <head>\n\t\t\t\t<meta charset=\"utf-8\">\n\t\t\t\t<title>" + req.body.id + "</title>\n\t\t\t\t<style>\n\t\t\t\tbody{\n\t\t\t\t\tbox-sizing: border-box;\n\t\t\t\t\tfont-family:arial;\n\t\t\t\t\tpadding:0 10px;\n\t\t\t\t\tfont-size:10px;\n\t\t\t\t  }\n\t\t\t\tdiv.in {\n\t\t\t\t\tbackground-color: #f0f8fb;\n\t\t\t\t\tcolor: #0f4979;\n\t\t\t\t\tborder: 1px solid #8ec8e0;\n\t\t\t\t\tpadding: 15px;\n\t\t\t\t\tmargin-bottom: 15px;\n\t\t\t\t}\n\t\t\t\t\n\t\t\t\tdiv.out{\n\t\t\t\t\tbackground-color: #f8faf5;\n\t\t\t\t\tcolor: #3f5c30;\n\t\t\t\t\tborder: 1px solid #aeca8d;\n\t\t\t\t\tpadding: 15px;\n\t\t\t\t\tmargin-bottom: 15px;\n\t\t\t\t}\n\t\t\t\tli.in[_ngcontent-c6] {\n\t\t\t\t\tbackground-color: #F0F8FB;\n\t\t\t\t\tcolor: #fff;\n\t\t\t\t}\n\t\t\t\t\n\t\t\t\tli.out {\n\t\t\t\t\tbackground-color: #EDF3E5;\n\t\t\t\t\tcolor: #fff;\n\t\t\t\t}\n\t\t\t\t.page-break {\n\t\t\t\t\tposition: relative;\n\t\t\t\t\theight: 100%;\n\t\t\t\t\twidth: 100%;\n\t\t\t\t\tdisplay: block;\n\t\t\t\t\tpage-break-after: always;\n\t\t\t\t\toverflow: hidden;\n\t\t\t\t  }\n\t\t\t\t</style>\n\t\t\t</head>\n\t\t\t<body>\n\t\t\t" + html + "\n\t\t\t</body>\n\t\t</html>";
        html_to_pdf.create(content, {
            format: 'A4',
            "header": {
                "height": "20mm",
                "contents": "<div style=\"text-align: center;display:flex\">" + (uri.length > 60 ? uri.substring(0, 58) : uri) + "...</div>"
            },
            "footer": {
                "height": "20mm",
                "contents": '<span style="color: #444;">{{page}}</span>/<span>{{pages}}</span>'
            }
        }).toBuffer(function (err, buffer) {
            if (err) {
                res.json({ status: false });
            }
            else {
                res.json({
                    status: true,
                    pdf: buffer.toString("base64")
                });
            }
        });
    });
    app.get('/api/evaluationplan/get_evaluationplan_by_name/:search_name', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_name = req.params.search_name || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Requesting evaluation plan by name.');
            return mssql_service_1.MSSql.GetEvaluationPlanByName(vcc, search_name);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Received evaluation plan data.');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error executing request.');
            logger.error('[EvaluationPlanService::get_evaluationplan_by_name] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationplan/get_evaluationplans/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplans] Received ' + req.method + ' post method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEvaluationPlans(vcc, search_criteria, search_expression, disabled);
        }).then(function (evaluationplans) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_summary/', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplans_summary] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var params = new evaluation_plan_model_1.EvaluationPlanFilters;
        var isAdmin = req.body.idAgent || false;
        var idAgent = req.body.idAgent || null;
        var idAuditor = req.body.idAuditor || null;
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationPlanService::get_evaluationplans_summary] Calling SP GetEvaluationPlansAgent from local DB');
            return mssql_service_1.MSSql.GetEvaluationPlansSummary(vcc, isAdmin, idAgent, idAuditor, params.campaigns.toString(), params.limitDate, params.evaluationFormats.toString(), params.modalities.toString(), params.completed);
        }).then(function (evaluationplans) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_summary] Received result from SP GetEvaluationPlansAgent from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_summary] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_auditor/:idAuditor', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var currentUserRole = res.locals.user.role;
        var currentUserId = res.locals.userId;
        var userFuntion = null;
        var params = new evaluation_plan_model_1.EvaluationPlanFilters;
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        params.view = req.body.view || null;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            logger.info('[EvaluationService::get_evaluationplans_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + vcc + '|' + currentUserId);
            switch (params.view) {
                case 'AUDITOR_INBOX':
                    userFuntion = 'auditor';
                    break;
                case 'BILLING_DETAIL_MODAL':
                    userFuntion = 'admin';
                    break;
                case 'EVALUATION':
                    userFuntion = 'admin';
                    break;
                default:
                    userFuntion = 'auditor';
                    break;
            }
            logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Calling SP GetEvaluationPlansAuditor from local DB');
            return mssql_service_1.MSSql.GetEvaluationPlansAuditor(vcc, currentUserId, params.campaigns.toString(), params.limitDate, params.evaluationPlans.toString(), params.evaluationFormats.toString(), params.modalities.toString(), params.completed, userFuntion);
        })
            .then(function (evaluationplans) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Received result from SP GetEvaluationPlansAuditor from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_auditor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_agent/:idAgent', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplans_agent] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var idAgent = res.locals.userId;
        var params = new evaluation_plan_model_1.EvaluationPlanFilters;
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationPlanService::get_evaluationplans_agent] Calling SP GetEvaluationPlansAgent from local DB');
            return mssql_service_1.MSSql.GetEvaluationPlansAgent(vcc, idAgent, params.campaigns.toString(), params.limitDate, params.evaluationFormats.toString(), params.modalities.toString(), params.completed);
        }).then(function (evaluationplans) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_agent] Received result from SP GetEvaluationPlansAgent from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/create_evaluationplan', function (req, res, next) {
        logger.info('[EvaluationPlanService::create_evaluationplan] Getting ' + req.method + ' method on \'' + req.path + '\'');
        var userId = res.locals.userId;
        var vcc = res.locals.vcc;
        var type = res.locals.type;
        var processAgentIds = [];
        var nowUtc = moment().utc().toDate();
        var newEvaluationPlan = (req.body || {});
        if (newEvaluationPlan.evaluatePercentage > 100)
            newEvaluationPlan.evaluatePercentage = newEvaluationPlan.evaluatePercentage % 100;
        if (newEvaluationPlan.evaluatePercentage > 1)
            newEvaluationPlan.evaluatePercentage = newEvaluationPlan.evaluatePercentage / 100;
        var params = new interaction_model_1.InteractionSearch;
        params = interaction_service_1.GetInteractionSearchFromRequest(req.body.idEvaluationFormat, newEvaluationPlan.interactionSearch, false);
        newEvaluationPlan.createdDate = nowUtc;
        newEvaluationPlan.createdByUserId = userId;
        newEvaluationPlan.lastModifiedDate = nowUtc;
        newEvaluationPlan.lastModifiedByUserId = userId;
        var channelInteractions = [];
        var totalSyncInteraction = 0;
        var totalAudio = 0;
        var audios = [];
        var totalSyncDetailPhrases = 0;
        var totalSyncDetailRawPhrases = 0;
        var asyncInteractions = [];
        var totalAsyncInteraction = 0;
        var totalAsyncDetail = 0;
        var totalAsyncDetailRawChar = 0;
        var totalAsyncDetailRawPhrases = 0;
        var totalAsyncDetailPhrases = 0;
        var hasAutomaticPart = false;
        if (newEvaluationPlan.evaluation_mode === 'A'
            || newEvaluationPlan.evaluation_mode === 'X'
            || (newEvaluationPlan.evaluation_mode === 'M'
                && newEvaluationPlan.filterByContentQuery
                && newEvaluationPlan.filterByContentQueryName))
            hasAutomaticPart = true;
        var batch = {};
        var billingVccRates = new billing_model_1.BillingVCCRates();
        var cognitiveService;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Checking if exists an EV by name::vcc -> ' + newEvaluationPlan.description + '::' + vcc);
            return mssql_service_1.MSSql.GetEvaluationPlanByName(vcc, newEvaluationPlan.description);
        })
            .then(function (result) {
            if (result)
                throw 'EVALUATION_PLAN_SERVER_ERROR_EXISTS';
            newEvaluationPlan.id = newEvaluationPlan.description;
            logger.info('[EvaluationPlanService::create_evaluationplan] The given name doesn\'t exists, saving the new plan.');
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                logger.info('[EvaluationPlanService::create_evaluationplan] Searching evaluation format\'s campaigns');
                return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
            }
            else {
                return bluebird_1.resolve(true);
            }
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Campaigns on evaluationFormat: ' + result.length);
            if (hasAutomaticPart) {
            }
            else {
                if (params.environmentType && (params.environmentType === 'allegro' || params.environmentType === 'i6')) {
                    if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                        if (!result || result.length === 0)
                            throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                        params.campaigns = result.map(function (r) { return r.id; });
                    }
                }
            }
            return Promise.resolve(true);
        }).then(function (result) {
            if (hasAutomaticPart) {
                return Promise.resolve(true);
            }
            else {
                if (params.environmentType
                    &&
                        (params.environmentType === 'allegro'
                            ||
                                params.environmentType === 'i6')) {
                    return Promise.resolve(true);
                }
                else {
                    var ftpCSVRedisKey = void 0;
                    if (params.evaluationFormatId)
                        ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
                    else
                        ftpCSVRedisKey = 'FTP_PROCESS:';
                    return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
                }
            }
        }).then(function (result) {
            if (params.environmentType !== 'allegro' && params.environmentType !== 'i6')
                if (result === null)
                    throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            var promiseArray = [];
            if (hasAutomaticPart) {
                promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsByProcessId(vcc, newEvaluationPlan.processId, 'RAW'));
            }
            else {
                logger.info('[EvaluationPlanService::create_evaluationplan] Requesting not evaluated interactions from HistoricalData');
                promiseArray = interaction_service_1.GetChannelNewInteractionSummaryPromises(vcc, type, false, 'DETAILED', params, result.data);
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Getting result for ' + result.length + ' channels');
            if (!result || result.length === 0)
                throw 'SERVER_ERROR_NO_INTEGRATED_DATA_FOUND';
            if (hasAutomaticPart) {
                newEvaluationPlan.channelInteractions = result[0];
                if (newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.Channel === 'CALL'; })) {
                    audios = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.Channel && ci.Channel === 'CALL'; });
                    totalSyncInteraction = audios.map(function (item) { return item['InteractionId']; }).filter(function (value, index, self) { return self.indexOf(value) === index; }).length;
                    totalAudio = audios.length;
                    totalSyncDetailPhrases = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.Channel && ci.Channel === 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.totalPhrases : a) + b.totalPhrases); }, 0) || 0;
                }
                else {
                    audios = [];
                    totalSyncInteraction = 0;
                    totalAudio = 0;
                    totalSyncDetailPhrases = 0;
                }
                if (newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel !== 'CALL'; })) {
                    asyncInteractions = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; });
                    totalAsyncInteraction = asyncInteractions.map(function (item) { return item['id']; }).filter(function (value, index, self) { return self.indexOf(value) === index; }).length;
                    totalAsyncDetail = newEvaluationPlan.channelInteractions.filter(function (r) { return r.channel && r.channel !== 'CALL'; }).length;
                    totalAsyncDetailRawChar = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.rawChar : a) + b.rawChar); }, 0);
                    totalAsyncDetailRawPhrases = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? Math.ceil(a.rawChar / cognitiveService.GetCognitiveEngineCharsPerPhrase()) : a) + Math.ceil(b.rawChar / cognitiveService.GetCognitiveEngineCharsPerPhrase())); }, 0);
                    totalAsyncDetailPhrases = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.totalPhrases : a) + b.totalPhrases); }, 0);
                }
                else {
                    asyncInteractions = [];
                    totalAsyncInteraction = 0;
                    totalAsyncDetail = 0;
                    totalAsyncDetailRawChar = 0;
                    totalAsyncDetailRawPhrases = 0;
                    totalAsyncDetailPhrases = 0;
                }
                var billing = new billing_implementation_1.BillingImplementation();
                return billing.GetVCCRates(vcc, newEvaluationPlan.processId);
            }
            else {
                channelInteractions = result;
                newEvaluationPlan.channelInteractions = [];
                var totalInteractionsToSave_1 = 0;
                channelInteractions.map(function (ci) { return totalInteractionsToSave_1 += ci && ci.totalInteractions ? ci.totalInteractions : 0; });
                totalInteractionsToSave_1 = Math.round(totalInteractionsToSave_1 * newEvaluationPlan.evaluatePercentage);
                channelInteractions.map(function (ci) {
                    if (ci && ci.totalInteractions) {
                        if (ci.interactionType === 'CALL') {
                            var iv = {};
                            iv = ci;
                            var idTitle = 'InteractionId';
                            var sampleResult = linguistic_service_1.GetInteractionsSample(iv.interactions, newEvaluationPlan.evaluatePercentage, idTitle, totalInteractionsToSave_1);
                            totalSyncInteraction = sampleResult.sampleInteractionIds.length;
                            audios = sampleResult.sampleInteractionDetails;
                            totalAudio = audios.length;
                            totalInteractionsToSave_1 -= sampleResult.sampleInteractionIds.length;
                            newEvaluationPlan.channelInteractions = newEvaluationPlan.channelInteractions.concat(audios);
                            audios.map(function (a) {
                                if (a.Agent)
                                    processAgentIds.push(a.Agent);
                            });
                        }
                        else {
                            var aiv = {};
                            aiv = ci;
                            asyncInteractions = [];
                            var idTitle = 'id';
                            var sampleResult = linguistic_service_1.GetInteractionsSample(aiv.interactions, newEvaluationPlan.evaluatePercentage, idTitle, totalInteractionsToSave_1);
                            totalAsyncInteraction += sampleResult.sampleInteractionIds.length;
                            asyncInteractions = sampleResult.sampleInteractionDetails;
                            totalAsyncDetail += asyncInteractions.length;
                            totalInteractionsToSave_1 -= sampleResult.sampleInteractionIds.length;
                            newEvaluationPlan.channelInteractions = newEvaluationPlan.channelInteractions.concat(asyncInteractions);
                            aiv.interactions.map(function (a) {
                                if (a.firstTakedAgent)
                                    processAgentIds.push(a.firstTakedAgent);
                                if (a.lastTakedAgent)
                                    processAgentIds.push(a.lastTakedAgent);
                            });
                        }
                    }
                });
                return true;
            }
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Receiving rates data');
            if (result === null)
                throw 'SERVER_ERROR_RATES_NOT_FOUND';
            var promiseArray = [];
            if (hasAutomaticPart) {
                billingVccRates.SetBillingVCCRates(result);
                batch.vcc = vcc;
                batch.totalInteraction = totalSyncInteraction + totalAsyncInteraction;
                batch.totalAsync = totalAsyncInteraction;
                batch.totalAsyncDetail = totalAsyncDetail;
                batch.totalSync = totalSyncInteraction;
                batch.totalAudio = totalAudio;
                batch.durationTime = 0;
                batch.roundedTime = 0;
                audios.map(function (a) {
                    batch.durationTime += a.DurationTime;
                    batch.roundedTime += Math.ceil(a.DurationTime / 60) * 60;
                });
                batch.totalQueries = 0;
                batch.totalModels = 0;
                batch.batchTypeName = 'EVALUATION_PLAN';
                batch.batchTypeId = newEvaluationPlan.id;
                batch.templateId = newEvaluationPlan.templateId;
                batch.processId = newEvaluationPlan.processId;
                batch.userApi = userId;
                batch.startDate = nowUtc;
                promiseArray.push(platform_service_1.GetPlatformVersion());
                if (batch.templateId) {
                    promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(batch.vcc, batch.templateId, null));
                    promiseArray.push(GetEvaluationPlanModels(batch.vcc, batch.templateId));
                }
                else {
                    promiseArray.push(Promise.resolve([]));
                    promiseArray.push(Promise.resolve([]));
                }
                promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(batch.vcc, newEvaluationPlan.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
                promiseArray.push(mssql_service_1.MSSql.GetAllegroBillingInteractionsForEvaluationPlan(batch.vcc, newEvaluationPlan.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
            }
            else {
                if (type !== 'OTHER')
                    promiseArray.push(linguistic_service_1.ImportAgents(vcc, type, processAgentIds));
                else
                    promiseArray.push(Promise.resolve(true));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Receiving extra data for batch');
            if (hasAutomaticPart) {
                var alreadyTranscribedAudio = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedAudio || 0;
                var alreadyDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedDurationTime || 0;
                var alreadyRoundedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedRoundedDurationTime || 0;
                var alreadyTranscribedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedValidDurationTime || 0;
                var alreadyTranscribedRoundedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedValidRoundedDurationTime || 0;
                totalSyncDetailRawPhrases = (batch.durationTime - alreadyDurationTime) / billingVccRates.cognitiveAnalysisAverageSecondsPerRequest;
                var estimatedRequests = {
                    modelRequestsFromCallTime: totalSyncDetailRawPhrases,
                    modelRequestFromCallAlreadyPhrases: totalSyncDetailPhrases,
                    modelRequestsFromTextRawChar: totalAsyncDetailRawPhrases,
                    modelRequestsFromTextAlreadyPhrases: totalAsyncDetailPhrases
                };
                if (result[0] === null)
                    throw 'INSTALLATIONID_MISSING';
                batch.totalCallPhrases = totalSyncDetailRawPhrases || 0;
                batch.alreadyCallPhrases = totalSyncDetailPhrases || 0;
                batch.totalTextPhrases = totalAsyncDetailRawPhrases || 0;
                batch.alreadyTextPhrases = totalAsyncDetailPhrases || 0;
                batch.startDate = nowUtc;
                batch.installationId = result[0].installationId;
                batch.totalQueries = (newEvaluationPlan.evaluation_mode === 'A' || newEvaluationPlan.evaluation_mode === 'X') ? result[1].length : 0;
                batch.totalModels = result[2].length;
                batch.modelRequests = billingVccRates.GetCognitiveAnalysisTotalAveragePhrasesFromNone(estimatedRequests);
                batch.alreadyTranscribedAudio = alreadyTranscribedAudio ? alreadyTranscribedAudio : 0;
                batch.alreadyDurationTime = alreadyDurationTime ? alreadyDurationTime : 0;
                batch.alreadyRoundedDurationTime = alreadyRoundedDurationTime ? alreadyRoundedDurationTime : 0;
                batch.alreadyTranscribedDurationTime = alreadyTranscribedDurationTime ? alreadyTranscribedDurationTime : 0;
                batch.alreadyTranscribedRoundedDurationTime = alreadyTranscribedRoundedDurationTime ? alreadyTranscribedRoundedDurationTime : 0;
                batch.inProcessAudio = result[4].inProcessAudio ? result[4].inProcessAudio : 0;
                batch.inProcessDurationTime = result[4].inProcessDurationTime ? result[4].inProcessDurationTime : 0;
                if (newEvaluationPlan.filterByContent
                    && newEvaluationPlan.filterByContentQuery
                    && newEvaluationPlan.filterByContentQueryName)
                    batch.totalQueries++;
                var billing = new billing_batch_implementation_1.BillingBatchImplementation();
                return billing.SaveBatchSummary(batch);
            }
            else {
                return { status: true };
            }
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Batch was saved.');
            if (result && result.status === false)
                throw result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
            return _this.SaveEvaluationPlan(vcc, newEvaluationPlan);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CREATE_EVALUATIONPLAN_FAILED';
            logger.info('[EvaluationPlanService::create_evaluationplan] The EV was saved on EvaluationPlan table.');
            return mssql_service_1.MSSql.BulkInteractionEvaluationPlan(vcc, newEvaluationPlan);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_SAVE_INTERACTION_EVALUATIONPLAN_FAILED';
            logger.info('[EvaluationPlanService::create_evaluationplan] getting result: ' + result + ' after BulkInteractionEvaluationPlan');
            var promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, newEvaluationPlan.id));
            if (newEvaluationPlan.filterByContentQuery
                && newEvaluationPlan.filterByContentQueryName) {
                logger.info('[EvaluationPlanService::create_evaluationplan] Adding into evaluationPlan queue.');
                promiseArray.push(_this.CreateQueueL(vcc, newEvaluationPlan.processId, 'evaluationPlan', newEvaluationPlan.id));
            }
            if (newEvaluationPlan.evaluation_mode === 'A'
                || newEvaluationPlan.evaluation_mode === 'X') {
                logger.info('[EvaluationPlanService::create_evaluationplan] Adding into process queue');
                promiseArray.push(_this.CreateQueueL(vcc, newEvaluationPlan.processId, 'ability', newEvaluationPlan.id));
            }
            if (hasAutomaticPart) {
                promiseArray.push(linguistic_service_1.BeginProcess(vcc, newEvaluationPlan.processId));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Calling function to save interactions per auditor.');
            newEvaluationPlan.uniqueInteractions = [];
            newEvaluationPlan.uniqueInteractions = result[0].reduce(function (acc, current) {
                var x = acc.find(function (item) { return item.id === current.id; });
                if (!x) {
                    return acc.concat([current]);
                }
                else {
                    if (current.dispositionCode) {
                        var arrDispositionCode = [];
                        arrDispositionCode = x.dispositionCode ? x.dispositionCode.split(',') : [];
                        if (!arrDispositionCode.includes(current.dispositionCode))
                            arrDispositionCode.push(current.dispositionCode);
                        acc.find(function (item) { return item.id === current.id; })['dispositionCode'] = arrDispositionCode.join(',');
                    }
                    return acc;
                }
            }, []);
            logger.info('[EvaluationPlanService::create_evaluationplan] UniqueInteractionIds: ' + newEvaluationPlan.uniqueInteractions.length);
            logger.info('[EvaluationPlanService::create_evaluationplan] UniqueInteractionIds: ' + (totalSyncInteraction + totalAsyncInteraction));
            logger.info('[EvaluationPlanService::create_evaluationplan] evaluation_mode::filterByContentQuery::filterByContentQueryName => ', newEvaluationPlan.evaluation_mode, '::', newEvaluationPlan.filterByContentQuery, '::', newEvaluationPlan.filterByContentQueryName);
            if ((newEvaluationPlan.evaluation_mode === 'A'
                ||
                    newEvaluationPlan.evaluation_mode === 'X')
                && newEvaluationPlan.filterByContentQuery === null
                && newEvaluationPlan.filterByContentQueryName === null) {
                logger.info('[EvaluationPlanService::create_evaluationplan] Asignando evaluaciones para auditor SPEECH');
                var auditor = {
                    idAuditor: 'SPEECH',
                    numberInteractions: newEvaluationPlan.uniqueInteractions.length
                };
                if (newEvaluationPlan.evaluation_mode === 'A')
                    newEvaluationPlan.auditors = [];
                newEvaluationPlan.auditors.push(auditor);
            }
            return SaveAuditorEP(vcc, newEvaluationPlan);
        }).then(function (resultSave) {
            if (resultSave === null)
                throw 'SERVER_ERROR_SAVE_AUDITOREP_FAILED';
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::create_evaluationplan] The EV save proccess has been finished.');
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::create_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
            _this.RemoveEvaluationPlanProcess(vcc, newEvaluationPlan);
        });
    });
    app.post('/api/evaluationplan/save_evaluationplan', function (req, res, next) {
        logger.info('[EvaluationPlanService::save_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var evaluationPlan = (req.body || {});
        evaluationPlan.createdDate = null;
        evaluationPlan.createdByUserId = null;
        evaluationPlan.lastModifiedDate = nowUtc;
        evaluationPlan.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            SaveEvaluationPlan(vcc, evaluationPlan);
        }).then(function (result) {
            return mssql_service_1.MSSql.DeleteAuditorEvaluationPlan(vcc, evaluationPlan.id);
        })
            .then(function (result) {
            evaluationPlan.evaluatePercentage = evaluationPlan.evaluatePercentage % 100;
            if (evaluationPlan.evaluatePercentage > 1)
                evaluationPlan.evaluatePercentage = evaluationPlan.evaluatePercentage / 100;
            if (evaluationPlan.evaluatePercentage < 1 && evaluationPlan.evaluatePercentage > 0) {
                return mssql_service_1.MSSql.SaveInteractionEvaluationPlanSample(vcc, evaluationPlan.id, evaluationPlan.evaluatePercentage);
            }
            else {
                return true;
            }
        }).then(function (result) {
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, evaluationPlan.id);
        })
            .then(function (result) {
            if (!result)
                throw 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE';
            evaluationPlan.interactions = result;
            evaluationPlan.uniqueInteractions = [];
            evaluationPlan.uniqueInteractions = result.reduce(function (acc, current) {
                var x = acc.find(function (item) { return item.id === current.id; });
                if (!x) {
                    return acc.concat([current]);
                }
                else {
                    return acc;
                }
            }, []);
            return SaveAuditorEP(vcc, evaluationPlan);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationPlanService::save_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationplan/get_evaluationplan/:evaluationplan_id', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlan = {};
        var auditorEvaluationPlan = [];
        var evaluationPlanId = req.params.evaluationplan_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!evaluationPlanId) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Evaluation plan ID is null');
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_ID';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Sending request to SP GetEvaluationPlan for EP ID: ' + evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationPlan(vcc, evaluationPlanId);
        }).then(function (result) {
            if (!result) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Evaluation plan doesn\'t exists on DB');
                throw 'EVALUATION_PLAN_SERVER_ERROR_CANNOT_GET';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Received evaluation plan from DB');
            evaluationPlan = result || evaluationPlan;
            return mssql_service_1.MSSql.GetAuditorsEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan] Received auditors from EP: ' + result.length);
            evaluationPlan.auditors = result || evaluationPlan.auditors;
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan] Received interaction details(audios/asyncinteractions): ' + result.length);
            evaluationPlan.interactions = result.reduce(function (acc, current) {
                var x = acc.find(function (item) { return item.id === current.id; });
                if (!x) {
                    return acc.concat([current]);
                }
                else {
                    return acc;
                }
            }, []);
            evaluationPlan.interactions = evaluationPlan.interactions || [];
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationPlan });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/delete_evaluationplan', function (req, res, next) {
        logger.info('[EvaluationPlanService::delete_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlan = (req.body || {});
        var evaluationPlanId = evaluationPlan.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!evaluationPlan)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(function (result) {
            if (!result)
                throw 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::delete_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/delete_evaluationplans', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlans = req.body.evaluationPlans || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!evaluationPlans)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            evaluationPlans.forEach(function (evaluationPlan) {
                promiseArray.push(mssql_service_1.MSSql.DeleteAuditorEvaluationPlan(vcc, evaluationPlan.id));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!evaluationPlans)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            evaluationPlans.forEach(function (evaluationPlan) {
                promiseArray.push(mssql_service_1.MSSql.DeleteEvaluationPlan(vcc, evaluationPlan.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeletes) {
            var result;
            if (resultDeletes.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Evaluation plans eliminados correctamente');
            }
            return result;
        }).then(function (resultDeletes) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeletes, description: (resultDeletes ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE'), data: resultDeletes });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.EvaluationPlanService = EvaluationPlanService;
function GetEvaluationPlanModels(vcc, templateId) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.GetTemplateThemeModels(vcc, templateId, null);
    }).then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_TEMPLATE_NLU_ITEMS_NOT_FOUND';
        var modelIntentIds = result.map(function (mi) { return mi.id; });
        return mssql_service_1.MSSql.GetUniqueNLUByIntents(vcc, templateId, modelIntentIds.join(','));
    }).then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_NLUS_NOT_FOUND';
        return Promise.resolve(result);
    }).catch(function (err) {
        return Promise.reject(err);
    });
}
function SaveEvaluationPlan(vcc, evaluationPlan) {
    return mssql_service_1.MSSql.SaveEvaluationPlan(vcc, evaluationPlan.id, evaluationPlan.modality, moment(evaluationPlan.dateFin).utc().toDate(), evaluationPlan.createdDate, evaluationPlan.createdByUserId, evaluationPlan.lastModifiedDate, evaluationPlan.lastModifiedByUserId, evaluationPlan.idEvaluationFormat, evaluationPlan.transcriptionLanguage, evaluationPlan.environmentId, evaluationPlan.description, evaluationPlan.enabled, evaluationPlan.filterByContent, evaluationPlan.filterByContentPercentage, evaluationPlan.evaluatePercentage, evaluationPlan.processAndEvaluate, evaluationPlan.filterByContentQueryName, evaluationPlan.filterByContentQuery, evaluationPlan.evaluationStarted, evaluationPlan.processId, evaluationPlan.status, evaluationPlan.evaluation_mode);
}
exports.SaveEvaluationPlan = SaveEvaluationPlan;
function SaveAuditorEP(vcc, evaluationPlan) {
    var promiseResult = true;
    return Promise.resolve(promiseResult)
        .then(function (result) {
        var promiseArray = [];
        logger.info('[EvaluationPlanService::SaveAuditorEP] Checking received auditors.');
        if (evaluationPlan.auditors) {
            var tmpInteractionsAll_1 = [];
            tmpInteractionsAll_1 = JSON.parse(JSON.stringify(evaluationPlan.uniqueInteractions));
            logger.info('[EvaluationPlanService::SaveAuditorEP] Received interactions: ' + tmpInteractionsAll_1.length);
            evaluationPlan.auditors.forEach(function (a) {
                var auditorInteractions = [];
                logger.info('[EvaluationPlanService::SaveAuditorEP] evaluationPlan.uniqueInteractions.length: ' + evaluationPlan.uniqueInteractions.length);
                logger.info('[EvaluationPlanService::SaveAuditorEP] auditor numberInteractions: ' + a.numberInteractions);
                if (evaluationPlan.uniqueInteractions.length === a.numberInteractions) {
                    auditorInteractions = evaluationPlan.uniqueInteractions;
                }
                else {
                    for (var i = 0; i < a.numberInteractions; i++) {
                        var randomIndex = Math.floor(Math.random() * tmpInteractionsAll_1.length);
                        auditorInteractions.push(tmpInteractionsAll_1[randomIndex]);
                        tmpInteractionsAll_1.splice(randomIndex, 1);
                    }
                }
                logger.info('[EvaluationPlanService::SaveAuditorEP] Asigned ' + auditorInteractions.length + 'interactions to ' + a.idAuditor);
                logger.info('[EvaluationPlanService::SaveAuditorEP] Making promises per auditor-interaction.');
                auditorInteractions.forEach(function (i) {
                    var interactionType = i.interactionType ? i.interactionType : i.Channel;
                    promiseArray.push(mssql_service_1.MSSql.SaveAuditorEvaluationPlan(vcc, evaluationPlan.id, a.idAuditor, interactionType === 'CALL' ? i.InteractionId || i.id : i.id, interactionType, interactionType === 'CALL' ? moment(i.dateIni).toDate() : moment(i.dateFin).toDate() || null, interactionType === 'CALL' ? moment(i.dateFin).toDate() : moment(i.dateFin).toDate() || null, interactionType === 'CALL' ? i.DurationTime || i.interactionTime : i.interactionTime || null, interactionType === 'CALL' ? i.Agent || i.idAgent || '' : i.agent || i.idAgent || '', interactionType === 'CALL' ? i.Campaign || i.idCampaign : i.campaign || i.idCampaign || '', interactionType === 'CALL' ? i.DispositionCode || i.dispositionCode : i.dispositionCode || null, interactionType === 'CALL' ? (i.Type && (i.Type === 'E' || i.origin === 'IN') ? 'IN' : 'OUT')
                        :
                            i.origin || null, interactionType === 'CALL' ? i.format : null, interactionType === 'CALL' ? i.contactName : i.contactName, evaluationPlan.lastModifiedByUserId));
                });
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (results) {
        var exitos = results.filter(function (result) { return (result === true); }).length;
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            logger.info('[EvaluationPlanService::SaveAuditorEP] Promises result OK.');
            promiseResult = false;
        }
        else {
            logger.info('[EvaluationPlanService::SaveAuditorEP] Promises result SERVER_ERROR_UNKNOWN_MESSAGE.');
            promiseResult = true;
        }
        return Promise.resolve(promiseResult);
    })
        .catch(function (err) {
        promiseResult = false;
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[EvaluationPlanService::SaveAuditorEP] Error: ' + errorMsg);
        return Promise.reject(promiseResult);
    });
}
exports.SaveAuditorEP = SaveAuditorEP;
function CreateQueueL(vcc, processId, typeName, typeId) {
    var currentQueueId = '';
    currentQueueId = vcc;
    currentQueueId += ':';
    currentQueueId += processId;
    if (typeName && typeId) {
        currentQueueId += ':';
        currentQueueId += typeName;
        currentQueueId += ':';
        currentQueueId += typeId;
    }
    return redis_service_1.Redis.CreateQueueL('Process', currentQueueId)
        .then(function (result) {
        if (result)
            return true;
        else
            return false;
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_CREATE_QUEUE');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[ProcessService::CreateQueue] getting errorMsg:', errorMsg);
    });
}
exports.CreateQueueL = CreateQueueL;
function RemoveEvaluationPlanProcess(vcc, evaluationPlan) {
    var myProcessId = evaluationPlan.processId;
    var promiseResult = true;
    return Promise.resolve(promiseResult)
        .then(function (result) {
        return mssql_service_1.MSSql.DeleteProcess(vcc, myProcessId);
    })
        .then(function (result) {
        if (!result)
            throw 'ERROR_DELETING_PROCESS';
        return Promise.resolve(promiseResult);
    })
        .catch(function (err) {
        promiseResult = false;
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[EvaluationPlanService::SaveAuditorEP] Error: ' + errorMsg);
        return Promise.reject(promiseResult);
    });
}
exports.RemoveEvaluationPlanProcess = RemoveEvaluationPlanProcess;
