"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetPlatformVersion = exports.PlatformService = void 0;
var log4js = require("log4js");
var config_1 = require("../config/config");
var promises_shared_1 = require("../shared/promises.shared");
var billing_implementation_1 = require("../implementation/billing.implementation");
var billing_account_implementation_1 = require("../implementation/billing-account.implementation");
var redis_service_1 = require("../db/redis.service");
var mssql_service_1 = require("../db/mssql.service");
var PJ = require('../../package.json');
var logger = log4js.getLogger('ServerScripts');
function PlatformService(app) {
    app.post('/api/activation/get_platform_status', function (req, res, next) {
        logger.info('[PlatformService::get_platform_status] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetPlatformStatus(vcc);
        }).then(function (result) {
            if (result === null)
                throw 'ERROR_NOT_RESULT_FROM_EXTERNAL_SERVER';
            var isValidated = result.platformStatus.isActivated ? true : false;
            result.platformCognitiveEngines = config_1.COGNITIVE_SERVICES_COGNITIVE_MOTORS;
            result.intentUmbral = config_1.INTENT_ACCEPTANCE;
            result.entityUmbral = config_1.ENTITY_ACCEPTANCE;
            logger.info('[PlatformService::get_platform_status] Result from external server: ' + result.description);
            if (res.headersSent)
                return;
            res.json({
                status: isValidated ? true : false,
                description: (isValidated ? 'SERVER_SUCCESS_MESSAGE' : 'PLATFORM_NOT_ACTIVATED'),
                data: result
            });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[PlatformService::get_platform_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/activation/get_platform_available_licenses', function (req, res, next) {
        logger.info('[PlatformService::get_platform_available_licenses] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetPlatformAvailableLicenses();
        }).then(function (result) {
            if (result === null)
                throw 'ERROR_NOT_RESULT_FROM_EXTERNAL_SERVER';
            logger.info('[PlatformService::get_platform_available_licenses] Result from external server: ' + result.description);
            if (res.headersSent)
                return;
            res.json({
                status: true,
                description: 'SERVER_SUCCESS_MESSAGE',
                data: result
            });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[PlatformService::get_platform_available_licenses] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/generate_license_file', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var guid = (req.body.guidContainer || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GenerateLicenseFileData(guid);
        }).then(function (result) {
            logger.info('[PlatformService::/api/generate_license_file] Getting result:', result, 'after billing.GenerateLicenseFileData');
            res.setHeader('Content-type', 'application/octet-stream');
            res.setHeader('Content-disposition', 'attachment; filename=license.lic');
            res.end(result);
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[PlatformService::/api/generate_license_file] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/activation/activate_platform', function (req, res, next) {
        logger.info('[PlatformService::activate_platform] Recived ' + req.method + ' on \'' + req.path + '\'');
        var params = (req.body || {});
        var account = params.user;
        var token = params.token;
        var user = params.user;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (token && user) {
                var billing = new billing_implementation_1.BillingImplementation();
                return billing.ActivatePlatform(account, token, user);
            }
            else {
                logger.info('[PlatformService::activate_platform] Received params: ', account, token, user);
                throw 'SERVER_ERROR_MISSING_SEARCH_PARAMETERS';
            }
        }).then(function (result) {
            logger.info('[PlatformService::activate_platform] Getting result:', result, 'after billing.ActivatePlatform');
            if (res.headersSent)
                return;
            var response = {};
            if (result && result.status === true) {
                response = { status: result.status, description: result.description, data: result.data };
            }
            else if (result && result.status === false) {
                response = { status: result.status, description: result.description, error: result.error };
            }
            else {
                response = { status: false, description: 'ERROR_NOT_RESULT_FROM_EXTERNAL_SERVER', error: 'ERROR_NOT_RESULT_FROM_EXTERNAL_SERVER' };
            }
            res.json(response);
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('PlatformService::get_platform_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/activation/get_platform_accounts', function (req, res) {
        logger.info('[PlatformService::get_platform_accounts] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetPlatformAccounts(vcc);
        }).then(function (result) {
            return Promise.resolve(result);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[PlatformService::/api/activation/get_platform_accounts] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_account_balance', function (req, res) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var account = req.body.account;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_account_implementation_1.BillingAccountImplementation();
            return billing.GetAccountBalance(account);
        }).then(function (result) {
            return Promise.resolve(result);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[PlatformService::/api/account/get_account_balances] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/platform/data_installation', function (req, res) {
        logger.info('[PlatformService::/api/platform/data_installation] Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var installationId = req.body.installationId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!installationId)
                throw 'MISSING_PARAMETER_INSTALATTION_ID';
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetInstallationData(installationId);
        }).then(function (result) {
            return Promise.resolve(result);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[PlatformService::/api/platform/data_installation] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.PlatformService = PlatformService;
function GetPlatformVersion() {
    var cacheUpdateNeeded = false;
    return redis_service_1.Redis.GetPlatformVersion()
        .then(function (result) {
        if (result
            && PJ
            && PJ.version
            && result
            && result.buildVersion
            && result.buildVersion === PJ.version
            && result.isRegistered) {
            return Promise.resolve(result);
        }
        else {
            logger.info('[PlatformService::GetPlatformVersion] Version data not found on redis');
            cacheUpdateNeeded = true;
            return mssql_service_1.MSSql.GetVersion();
        }
    }).then(function (result) {
        if (result) {
            if (cacheUpdateNeeded)
                redis_service_1.Redis.SavePlatformVersion(result);
            return Promise.resolve(result);
        }
        else {
            throw 'SERVER_ERROR_PLATFORM_VERSION_NOT_FOUND_ON_DATABASE';
        }
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[PlatformService::GetPlatformVersion] Error on: ' + errMsg);
    });
}
exports.GetPlatformVersion = GetPlatformVersion;
