"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TopWorstAgentsService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function TopWorstAgentsService(app) {
    app.post('/api/top_worst_agents/get_top_worst_agents/', function (req, res, next) {
        logger.info('[TopWorstAgentsService::get_top_worst_agents] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var initialDate = req.body.initialDate ? moment(req.body.initialDate).toDate() : null;
        var endDate = req.body.endDate ? moment(req.body.endDate).toDate() : null;
        var campaigns = (req.body.campaigns && req.body.campaigns !== undefined) ? req.body.campaigns : null;
        var evaluationFormats = (req.body.evaluationFormats && req.body.evaluationFormats !== undefined) ? req.body.evaluationFormats : null;
        var evaluationPlans = (req.body.evaluationPlans && req.body.evaluationPlans !== undefined) ? req.body.evaluationPlans : null;
        var agents = (req.body.agents && req.body.agents !== undefined) ? req.body.agents : null;
        var idGroup = (req.body.idGroup && req.body.idGroup !== 0) ? req.body.idGroup : null;
        var idSubGroup = (req.body.idSubGroup && req.body.idSubGroup !== 0) ? req.body.idSubGroup : null;
        var idAbility = (req.body.idAbility && req.body.idAbility !== 0) ? req.body.idAbility : null;
        var auditors = (req.body.auditors && req.body.auditors !== undefined) ? req.body.auditors : null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (initialDate && endDate) {
                return mssql_service_1.MSSql.GetTopWorstAgents(instance, initialDate, endDate, 10, campaigns, evaluationFormats, evaluationPlans, agents, idGroup, idSubGroup, idAbility, auditors);
            }
            else {
                return [];
            }
        }).then(function (resultTopPeoresAgentes) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: (resultTopPeoresAgentes ? resultTopPeoresAgentes : []) });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TopWorstAgentsService::get_top_worst_agents] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TopWorstAgentsService = TopWorstAgentsService;
