"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var express = require("express");
var moment = require("moment");
var log4js = require("log4js");
var mssql_config_1 = require("../config/mssql.config");
var http_request_occ_1 = require("../shared/http-request-occ");
var mssql_service_1 = require("../db/mssql.service");
var mssql_init_1 = require("../db/mssql.init");
var integration_config_1 = require("../config/integration.config");
var TYPE = '5.5';
var loggerAllegro = log4js.getLogger('Task');
function ExecuteTask() {
    loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de Usuarios');
    var users = [];
    var usersToSave;
    var nowUtc = moment().utc().toDate();
    var request = {};
    request.columns = ["Id", "Mail", "Password", "UserGroup", "Name", "FirstSurname", "LastName", "LastSurname", "LanguageId", "countryCode", "TimeZoneId"];
    request.limit = 1000;
    if (TYPE === 'OCC' || TYPE === 'INTEGRATED') {
        loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de Usuarios de i6');
        return mssql_service_1.MSSql.GetAPIUsersOCC()
            .then(function (apiUsers) {
            if (apiUsers === [])
                return Promise.resolve(null);
            var promiseArray = [];
            var vcc;
            apiUsers.forEach(function (apiUser) {
                promiseArray.push(http_request_occ_1.Login(apiUser.user + '@' + apiUser.vcc, apiUser.password)
                    .then(function (resultLogin) {
                    if (!resultLogin.accessToken)
                        return Promise.resolve(null);
                    return http_request_occ_1.SearchUsers(resultLogin.accessToken, request);
                }).then(function (resultSearchUsers) {
                    if (!resultSearchUsers.status && resultSearchUsers.rows === [])
                        return Promise.resolve(null);
                    users = [];
                    usersToSave = [];
                    usersToSave = resultSearchUsers.rows.filter(function (resultSearchUsers) { return resultSearchUsers.UserGroup.toLowerCase() === 'agent' || resultSearchUsers.UserGroup.toLowerCase() === 'qualityauditor'; });
                    for (var _i = 0, usersToSave_1 = usersToSave; _i < usersToSave_1.length; _i++) {
                        var row = usersToSave_1[_i];
                        var user = {};
                        user.vcc = apiUser.vcc;
                        user.id = row.Id;
                        user.email = row.Mail ? row.Mail : '';
                        user.firstname = row.Name + (row.LastName ? ' ' + row.LastName : '');
                        user.lastname = row.FirstSurname + (row.LastSurname ? ' ' + row.LastSurname : '');
                        user.password = row.Password ? row.Password : '';
                        user.role = row.UserGroup ? row.UserGroup : '';
                        user.country = row.countryCode ? row.countryCode : '';
                        user.language = row.LanguageId ? row.LanguageId : '';
                        user.timeZone = row.TimeZoneId ? row.TimeZoneId : '';
                        user.preferences = null;
                        user.isSystemUser = false;
                        user.limitClients = null;
                        user.createdDate = nowUtc;
                        user.createdByUserId = 'admin';
                        user.lastModifiedDate = nowUtc;
                        user.lastModifiedByUserId = 'admin';
                        users.push(user);
                    }
                    return Promise.resolve(users);
                }).then(function (usersToSave) {
                    if (usersToSave === [])
                        return Promise.resolve(null);
                    return ExecuteCreateUser(usersToSave);
                }).catch(function (err) {
                    loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
                    return Promise.resolve(null);
                }));
            });
            return Promise.all(promiseArray).then(function (results) {
                if (results.filter(function (result) { return (result !== true); }).length > 0) {
                    return false;
                }
                else {
                    return true;
                }
            });
        }).catch(function (err) {
            loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
            return Promise.resolve(null);
        });
    }
    else if (TYPE === '5.5') {
        loggerAllegro.info('ExecuteTask (inConcert): Inicia sincronizacion de Usuarios de Allegro');
        var flag = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
        return mssql_service_1.MSSql.GetIntegrationUsers(integration_config_1.IP_BD, flag)
            .then(function (usersToSave) {
            if (usersToSave === [])
                return Promise.resolve(null);
            return ExecuteCreateUser(usersToSave);
        }).catch(function (err) {
            loggerAllegro.error('ExecuteTask (inConcert): Error: ' + err.message);
            return Promise.resolve(null);
        });
    }
    else {
        loggerAllegro.error('ExecuteTask (inConcert): No cumplio con el tipo de ambiente');
        return Promise.resolve(null);
    }
}
function ExecuteCreateUser(users) {
    var promiseArray = [];
    if (users) {
        users.forEach(function (user) {
            promiseArray.push(mssql_service_1.MSSql.SaveIntegratedUser(user.vcc, user.id, user.email, user.firstname, user.lastname, user.password, user.role.toLowerCase() === 'agent' ? 'quality_agent' : (user.role.toLowerCase() === 'qualityauditor' ? 'quality_auditor' : ''), user.country, user.language, user.timeZone, user.preferences, user.isSystemUser, user.limitClients, user.createdDate, user.createdByUserId, user.lastModifiedDate, user.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
if (require.main === module) {
    var app_1 = express();
    var sqlConn_1;
    mssql_config_1.MSSQL_CONFIG.connectionTimeout = 300 * 1000;
    mssql_config_1.MSSQL_CONFIG.requestTimeout = 300 * 1000;
    mssql_init_1.MsSqlInit(app_1)
        .then(function (result) {
        sqlConn_1 = app_1.get('sqlConn');
        return ExecuteTask();
    }).then(function (results) {
        loggerAllegro.info('Task (allegro): Termino la sincronizacion de usuarios');
        sqlConn_1.close();
        setTimeout(function () {
            process.exit(0);
        }, 200);
    }).catch(function (error) {
        loggerAllegro.error('Task (allegro): Termino la sincronizacion de usuarios con errores :', error);
        sqlConn_1.close();
        setTimeout(function () {
            process.exit(0);
        }, 200);
    });
}
